@@ -238,6 +238,9 @@ class HitObject:
         self.time = time
         self.hitsound = hitsound
         self.addition = addition
+        self.ht_enabled = False
+        self.dt_enabled = False
+        self.hr_enabled = False
 
     def __repr__(self):
         return (
@@ -274,20 +277,33 @@ class HitObject:
         """The ``HitObject`` as it would appear with
         :data:`~slider.mod.Mod.half_time` enabled.
         """
-        return self._time_modify(4 / 3)
+        if self.ht_enabled:
+            return self
+
+        obj = self._time_modify(4 / 3)
+        obj.ht_enabled = True
+        return obj
 
     @lazyval
     def double_time(self):
         """The ``HitObject`` as it would appear with
         :data:`~slider.mod.Mod.double_time` enabled.
         """
-        return self._time_modify(2 / 3)
+        if self.dt_enabled:
+            return self
+
+        obj = self._time_modify(2 / 3)
+        obj.dt_enabled = True
+        return obj
 
     @lazyval
     def hard_rock(self):
         """The ``HitObject`` as it would appear with
         :data:`~slider.mod.Mod.hard_rock` enabled.
         """
+        if self.hr_enabled:
+            return self
+
         kwargs = {}
         for name in inspect.signature(type(self)).parameters:
             value = getattr(self, name)
@@ -295,7 +311,9 @@ class HitObject:
                 value = Position(value.x, 384 - value.y)
             kwargs[name] = value
 
-        return type(self)(**kwargs)
+        obj = type(self)(**kwargs)
+        obj.hr_enabled = True
+        return obj
 
     @classmethod
     def parse(cls, data, timing_points, slider_multiplier, slider_tick_rate):
@@ -329,12 +347,20 @@ class HitObject:
             raise ValueError(f'not enough elements in line, got {data!r}')
 
         try:
-            x = int(x)
+            # in old beatmaps (and potentially newer ones which were manually
+            # edited?), x and y can be floats (see b/128). Without the
+            # secondary float cast, parsing these maps would fail.
+            # Lazer casts these to integers
+            # (https://github.com/ppy/osu/blob/d4ea57c6607a77abb8a5e2fe55b220d8
+            # dfeeb456/osu.Game/Rulesets/Objects/Legacy/ConvertHitObjectParser.
+            # cs#L49), so we're still matching in-game positions even though we
+            # technically lose precision from the .osu file by casting.
+            x = int(float(x))
         except ValueError:
             raise ValueError(f'x should be an int, got {x!r}')
 
         try:
-            y = int(y)
+            y = int(float(y))
         except ValueError:
             raise ValueError(f'y should be an int, got {y!r}')
 
@@ -550,6 +576,9 @@ class Slider(HitObject):
         """The ``HitObject`` as it would appear with
         :data:`~slider.mod.Mod.hard_rock` enabled.
         """
+        if self.hr_enabled:
+            return self
+
         kwargs = {}
         for name in inspect.signature(type(self)).parameters:
             value = getattr(self, name)
@@ -558,8 +587,9 @@ class Slider(HitObject):
             elif name == 'curve':
                 value = value.hard_rock
             kwargs[name] = value
-
-        return type(self)(**kwargs)
+        obj = type(self)(**kwargs)
+        obj.hr_enabled = True
+        return obj
 
     @classmethod
     def _parse(cls,
@@ -1232,8 +1262,8 @@ class Beatmap:
         self.slider_tick_rate = slider_tick_rate
         self.timing_points = timing_points
         self._hit_objects = hit_objects
-        # cache hit objects once stacking is calculated
-        self._hit_objects_with_stacking = None
+        # cache hit object stacking at different ar and cs values
+        self._hit_objects_with_stacking = {}
 
         # cache the stars with different mod combinations
         self._stars_cache = {}
@@ -1466,18 +1496,22 @@ class Beatmap:
         if stacking:
             ar = self.ar(easy=easy, hard_rock=hard_rock)
             cs = self.cs(easy=easy, hard_rock=hard_rock)
+            # stacking changes with ar and cs (or equivalently EZ/HR), so only
+            # cache up to ar and cs
+            stacking_key = (ar, cs)
+
             if self.format_version >= 6:
                 resolve_stacking_method = self._resolve_stacking
             else:
                 resolve_stacking_method = self._resolve_stacking_old
 
             # use cache if available
-            if self._hit_objects_with_stacking:
-                hit_objects = self._hit_objects_with_stacking
+            if stacking_key in self._hit_objects_with_stacking:
+                hit_objects = self._hit_objects_with_stacking[stacking_key]
             else:
                 hit_objects = resolve_stacking_method(hit_objects, ar, cs)
                 # cache stacking calculation
-                self._hit_objects_with_stacking = hit_objects
+                self._hit_objects_with_stacking[stacking_key] = hit_objects
 
         if double_time:
             hit_objects = [ob.double_time for ob in hit_objects]
@@ -1676,6 +1710,60 @@ class Beatmap:
 
         return hit_objects
 
+    @lazyval
+    def _hit_object_times(self):
+        """a (sorted) list of hitobject time's, so they can be searched with
+        ``np.searchsorted``
+        """
+        return [hitobj.time for hitobj in self._hit_objects]
+
+    def closest_hitobject(self, t, side="left"):
+        """The hitobject closest in time to ``t``.
+
+        Parameters
+        ----------
+        t : datetime.timedelta
+            The time to find the hitobject closest to.
+        side : {"left", "right"}
+            Whether to prefer the earlier (left) or later (right) hitobject
+            when breaking ties.
+
+        Returns
+        -------
+        hit_object : HitObject
+            The closest hitobject in time to ``t``.
+        None
+            If the beatmap has no hitobjects.
+        """
+        if len(self._hit_objects) == 0:
+            raise ValueError(f"The beatmap {self!r} must have at least one "
+                             "hit object to determine the closest hitobject.")
+        if len(self._hit_objects) == 1:
+            return self._hit_objects[0]
+
+        i = np.searchsorted(self._hit_object_times, t)
+        # if ``t`` is after the last hitobject, an index of
+        # len(self._hit_objects) will be returned. The last hitobject will
+        # always be the closest hitobject in this case.
+        if i == len(self._hit_objects):
+            return self._hit_objects[-1]
+        # similar logic follows for the first hitobject.
+        if i == 0:
+            return self._hit_objects[0]
+
+        # searchsorted tells us the two closest hitobjects, but not which is
+        # closer. Check both candidates.
+        hitobj1 = self._hit_objects[i - 1]
+        hitobj2 = self._hit_objects[i]
+        dist1 = abs(hitobj1.time - t)
+        dist2 = abs(hitobj2.time - t)
+
+        hitobj1_closer = dist1 <= dist2 if side == "left" else dist1 < dist1
+
+        if hitobj1_closer:
+            return hitobj1
+        return hitobj2
+
     @lazyval
     def max_combo(self):
         """The highest combo that can be achieved on this beatmap.
@@ -1840,6 +1928,10 @@ class Beatmap:
             group_buffer = []
 
         for line in lines:
+            # some (presmuably manually edited) beatmaps have whitespace at the
+            # beginning or end of lines. This can cause logic relying on tokens
+            # occurring at specific indices to fail, so we get rid of it.
+            line = line.strip()
             if not line or line.startswith('//'):
                 # filter out empty lines and comments
                 continue
@@ -1925,10 +2017,10 @@ class Beatmap:
                 milliseconds=_get_as_int(groups, 'General', 'AudioLeadIn', 0),
             ),
             preview_time=timedelta(
-                milliseconds=_get_as_int(groups, 'General', 'PreviewTime'),
+                milliseconds=_get_as_int(groups, 'General', 'PreviewTime', -1),
             ),
             countdown=_get_as_bool(groups, 'General', 'Countdown', False),
-            sample_set=_get_as_str(groups, 'General', 'SampleSet'),
+            sample_set=_get_as_str(groups, 'General', 'SampleSet', 'Normal'),
             stack_leniency=_get_as_float(
                 groups,
                 'General',
