@@ -1,7 +1,6 @@
 import datetime
 from enum import IntEnum, unique
 
-import pytz
 import requests
 
 from .game_mode import GameMode
@@ -523,18 +522,19 @@ class Client:
         'passcount': 'pass_count',
     }
 
-    def _parse_date(cs, *, _tz=pytz.FixedOffset(8 * 60)):
-        # _tz is UTC+8
-        if cs is None:
-            return None
-
-        return _tz.localize(
-            datetime.datetime.strptime(cs, '%Y-%m-%f %H:%M:%S'),
-        )
+    def _parse_date(cs):
+        return datetime.datetime.strptime(cs, '%Y-%m-%f %H:%M:%S')
 
     def _parse_timedelta(cs):
         return datetime.timedelta(seconds=int(cs))
 
+    def _parse_optional(class_):
+        def func(cs):
+            if cs is None:
+                return cs
+            return class_(cs)
+        return func
+
     def _identity(cs):
         return cs
 
@@ -552,7 +552,7 @@ class Client:
         'favourite_count': int,
         'play_count': int,
         'pass_count': int,
-        'max_combo': lambda cs: cs if cs is None else int(cs),
+        'max_combo': _parse_optional(int),
         'title': _identity,
         'version': _identity,
     }
@@ -685,7 +685,9 @@ class Client:
     def _parse_user_events(events, _parse_date=_parse_date):
         out = []
         for event in events:
-            event['beatmap_id'] = int(event['beatmap_id'])
+            # beatmap id can be null if the event is a supporter gift
+            if event['beatmap_id'] is not None:
+                event['beatmap_id'] = int(event['beatmap_id'])
             event['date'] = _parse_date(event['date'])
             event['epic_factor'] = event.pop('epicfactor')
             out.append(event)
@@ -695,21 +697,23 @@ class Client:
     _user_conversions = {
         'user_id': int,
         'user_name': _identity,
-        'count_300': int,
-        'count_100': int,
-        'count_50': int,
-        'play_count': int,
-        'ranked_score': int,
-        'total_score': int,
-        'pp_rank': int,
-        'level': float,
-        'pp_raw': float,
-        'accuracy': float,
-        'count_ss': int,
-        'count_s': int,
-        'count_a': int,
+        # these attributes can be null for users that have never played before,
+        # see user 17906393 / #97 on github
+        'count_300': _parse_optional(int),
+        'count_100': _parse_optional(int),
+        'count_50': _parse_optional(int),
+        'play_count': _parse_optional(int),
+        'ranked_score': _parse_optional(int),
+        'total_score': _parse_optional(int),
+        'pp_rank': _parse_optional(int),
+        'level': _parse_optional(float),
+        'pp_raw': _parse_optional(float),
+        'accuracy': _parse_optional(float),
+        'count_ss': _parse_optional(int),
+        'count_s': _parse_optional(int),
+        'count_a': _parse_optional(int),
         'country': _identity,
-        'pp_country_rank': int,
+        'pp_country_rank': _parse_optional(int),
         'events': _parse_user_events,
     }
 
